@@ -109,7 +109,7 @@ typedef struct
     GtkWidget *update_spinner;
     
     /* Network device */
-    GtkWidget *net_entry;
+    GtkWidget *net_combo;
     
     /* Maximum */
     GtkWidget *max_use_label;
@@ -149,8 +149,9 @@ typedef struct
 
 static void set_progressbar_csscolor(GtkWidget*, GdkRGBA*);
 /* ---------------------------------------------------------------------------------------------- */
-static gboolean update_monitors(t_global_monitor *global)
+static gboolean update_monitors(gpointer user_data)
 {
+   t_global_monitor *global = user_data;
     char buffer[SUM+1][BUFSIZ];
     char buffer_panel[SUM][BUFSIZ];
     gchar caption[BUFSIZ];
@@ -180,7 +181,7 @@ static gboolean update_monitors(t_global_monitor *global)
         /* correct value to be from 1 ... 100 */
         global->monitor->history[i][0] = net[i];
 
-        if (global->monitor->history[i][0] < 0)
+        if ((gint) global->monitor->history[i][0] < 0)
         {
             global->monitor->history[i][0] = 0;
         }
@@ -284,8 +285,7 @@ static void run_update (t_global_monitor *global)
 
     if (global->monitor->options.update_interval > 0)
     {
-        global->timeout_id =  g_timeout_add( global->monitor->options.update_interval, 
-            (GSourceFunc)update_monitors, global);
+        global->timeout_id = g_timeout_add(global->monitor->options.update_interval, update_monitors, global);
     }
 }
 
@@ -538,11 +538,11 @@ static void set_progressbar_csscolor(GtkWidget* pbar, GdkRGBA* color)
 {
     gchar * css;
 #if GTK_CHECK_VERSION (3, 20, 0)
-    css = g_strdup_printf("progressbar progress { background-color: %s; background-image: none; }",
+    css = g_strdup_printf("progressbar progress { background-color: %s; border-color: %s; background-image: none; }",
 #else
-    css = g_strdup_printf(".progressbar progress { background-color: %s; background-image: none; }",
+    css = g_strdup_printf(".progressbar progress { background-color: %s; border-color: %s; background-image: none; }",
 #endif
-                          gdk_rgba_to_string(color));
+                          gdk_rgba_to_string(color), gdk_rgba_to_string(color));
     DBG("setting pbar css to %s", css);
     gtk_css_provider_load_from_data (g_object_get_data(G_OBJECT(pbar),"css_provider"), css, strlen(css), NULL);
     g_free(css);
@@ -791,7 +791,7 @@ static void monitor_apply_options(t_global_monitor *global)
         g_free(global->monitor->options.network_device);
     }
     global->monitor->options.network_device =
-        g_strdup(gtk_entry_get_text(GTK_ENTRY(global->monitor->net_entry)));
+        gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(global->monitor->net_combo));
     
     for( i = 0; i < SUM; i++ )
     {
@@ -839,22 +839,6 @@ static void max_label_changed(GtkWidget *button, t_global_monitor *global)
 }
 
 
-/* ---------------------------------------------------------------------------------------------- */
-static void network_changed(GtkWidget *button, t_global_monitor *global)
-{
-    if (global->monitor->options.network_device)
-    {
-        g_free(global->monitor->options.network_device);
-    }
-
-    global->monitor->options.network_device =
-        g_strdup(gtk_entry_get_text(GTK_ENTRY(global->monitor->net_entry)));
-
-    setup_monitor(global, FALSE);
-    DBG("network_changed");
-}
-
-
 /* ---------------------------------------------------------------------------------------------- */
 static void label_toggled(GtkWidget *check_button, t_global_monitor *global)
 {
@@ -956,11 +940,78 @@ static void change_color_out(GtkWidget *button, t_global_monitor *global)
 static void
 monitor_dialog_response (GtkWidget *dlg, int response, t_global_monitor *global)
 {
+    if (response == GTK_RESPONSE_HELP) {
+        xfce_dialog_show_help (GTK_WINDOW (dlg), PACKAGE_VERSION, NULL, NULL);
+    }
+    else {
+        monitor_apply_options (global);
+        gtk_widget_destroy (dlg);
+        xfce_panel_plugin_unblock_menu (global->plugin);
+        monitor_write_config (global->plugin, global);
+    }
+}
 
-    monitor_apply_options (global);
-    gtk_widget_destroy (dlg);
-    xfce_panel_plugin_unblock_menu (global->plugin);
-    monitor_write_config (global->plugin, global);
+static void
+monitor_show_about (XfcePanelPlugin *plugin, t_global_monitor *global)
+{
+    const gchar *auth[] = {
+      "Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>",
+      "Bernhard Walle <bernhard.walle@gmx.de>",
+      "Hendrik Scholz <hscholz@raisdorf.net> (Wormulon code base)",
+      "Florian Rivoal <frivoal@xfce.org>",
+      "Simon Steinbeiß <simon@xfce.org>", NULL };
+
+    gtk_show_about_dialog (NULL,
+      "logo-icon-name", "org.xfce.panel.netload",
+      "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
+      "version", PACKAGE_VERSION,
+      "program-name", PACKAGE_NAME,
+      "comments", _("Monitor CPU load, swap usage and memory footprint"),
+      "website", "https://docs.xfce.org/panel-plugins/xfce4-netload-plugin/start",
+      "copyright", _("Copyright (c) 2003-2021\n"),
+      "authors", auth, NULL);
+}
+
+static gboolean add_interface(const gchar *name, gpointer ignore, t_global_monitor *global)
+{
+    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(global->monitor->net_combo), name);
+    if (!strcmp(name, global->monitor->options.network_device)) {
+        GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(global->monitor->net_combo));
+        gint n_items = gtk_tree_model_iter_n_children(model, NULL);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(global->monitor->net_combo), n_items - 1);
+    }
+
+    return FALSE;
+}
+
+static gint monitor_populate_interface_list(t_global_monitor *global)
+{
+    gint   count = 0;
+#if defined(__linux__) || defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__NetBSD__)
+    struct ifaddrs *addrs = NULL, *addr;
+
+    if (getifaddrs(&addrs) == 0)
+    {
+        GTree *found_ifaces = g_tree_new((GCompareFunc) strcmp);
+
+        for (addr = addrs; addr != NULL; addr = addr->ifa_next)
+        {
+            if (addr->ifa_name != NULL && g_tree_lookup(found_ifaces, addr->ifa_name) == NULL)
+            {
+                DBG("Found interface: %s", addr->ifa_name);
+                g_tree_insert(found_ifaces, addr->ifa_name, GUINT_TO_POINTER(TRUE));
+                ++count;
+            }
+        }
+
+        g_tree_foreach(found_ifaces, (GTraverseFunc) add_interface, global);
+
+        g_tree_unref(found_ifaces);
+        freeifaddrs(addrs);
+    }
+#endif /* __linux__ || __OpenBSD__ || __FreeBSD__ || __NetBSD__ */
+
+    return count;
 }
 
 static void monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
@@ -975,37 +1026,38 @@ static void monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *gl
     GtkWidget        *color_label[SUM];
     gint             present_data_active;
     GtkSizeGroup     *sg;
-    gint             i;
+    gint             i, n_interfaces;
     gchar            buffer[BUFSIZ];
-    gchar            *color_text[] = { 
-                        N_("Bar color (i_ncoming):"), 
-                        N_("Bar color (_outgoing):") 
+    gchar            *color_text[] = {
+                        N_("Bar color (i_ncoming):"),
+                        N_("Bar color (_outgoing):")
                      };
     gchar            *maximum_text_label[] = {
                         N_("Maximum (inco_ming):"),
                         N_("Maximum (o_utgoing):")
                      };
-    
+
     xfce_panel_plugin_block_menu (plugin);
-    
-    dlg = xfce_titled_dialog_new_with_buttons (_("Network Monitor"), NULL,
-                                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                                               "gtk-close", GTK_RESPONSE_OK,
-                                               NULL);
-    
-    gtk_window_set_icon_name (GTK_WINDOW (dlg), "xfce4-settings");
+
+    dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Network Monitor"), NULL,
+                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     "window-close-symbolic", _("_Close"), GTK_RESPONSE_OK,
+                                                     "help-browser", _("_Help"), GTK_RESPONSE_HELP,
+                                                     NULL);
+
+    gtk_window_set_icon_name (GTK_WINDOW (dlg), "org.xfce.panel.netload");
     g_signal_connect (dlg, "response", G_CALLBACK (monitor_dialog_response),
                       global);
 
     global->opt_dialog = dlg;
-    
+
     global_vbox = GTK_BOX (gtk_box_new(GTK_ORIENTATION_VERTICAL, 6));
     gtk_container_set_border_width (GTK_CONTAINER (global_vbox), 12);
     gtk_widget_show(GTK_WIDGET (global_vbox));
     gtk_box_pack_start(GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dlg))), GTK_WIDGET (global_vbox), TRUE, TRUE, 0);
-    
+
     sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-    
+
     vbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 6));
     gtk_widget_show(GTK_WIDGET(vbox));
 
@@ -1055,18 +1107,21 @@ static void monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *gl
     gtk_box_pack_start(GTK_BOX(net_hbox), GTK_WIDGET(device_label),
                        FALSE, FALSE, 0);
 
-    global->monitor->net_entry = gtk_entry_new();
+    global->monitor->net_combo = gtk_combo_box_text_new_with_entry();
+    n_interfaces = monitor_populate_interface_list(global);
     gtk_label_set_mnemonic_widget(GTK_LABEL(device_label),
-                                  global->monitor->net_entry);
-    gtk_entry_set_max_length(GTK_ENTRY(global->monitor->net_entry),
-                             MAX_LENGTH);
-    gtk_entry_set_text(GTK_ENTRY(global->monitor->net_entry),
-                       global->monitor->options.network_device);
-    gtk_widget_show(global->monitor->opt_entry);
-
-    gtk_box_pack_start(GTK_BOX(net_hbox), GTK_WIDGET(global->monitor->net_entry),
+                                  global->monitor->net_combo);
+    if (gtk_combo_box_get_active(GTK_COMBO_BOX(global->monitor->net_combo)) == -1 &&
+        global->monitor->options.network_device[0] != '\0')
+    {
+        gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(global->monitor->net_combo),
+                                       global->monitor->options.network_device);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(global->monitor->net_combo), n_interfaces);
+    }
+    gtk_widget_show(global->monitor->net_combo);
+    gtk_box_pack_start(GTK_BOX(net_hbox), GTK_WIDGET(global->monitor->net_combo),
                        FALSE, FALSE, 0);
-    
+
     gtk_size_group_add_widget(sg, device_label);
 
     gtk_widget_show_all(GTK_WIDGET(net_hbox));
@@ -1267,8 +1322,6 @@ static void monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *gl
             G_CALLBACK(label_changed), global);
     g_signal_connect(GTK_WIDGET(global->monitor->opt_present_data_combobox), "changed",
             G_CALLBACK(present_data_combobox_changed), global);
-    g_signal_connect(GTK_WIDGET(global->monitor->net_entry), "activate",
-            G_CALLBACK(network_changed), global);
     g_signal_connect(GTK_WIDGET(global->monitor->opt_as_bits), "toggled",
             G_CALLBACK(as_bits_toggled), global);
 
@@ -1286,20 +1339,23 @@ static void netload_construct (XfcePanelPlugin *plugin)
     global = monitor_new(plugin);
 
     monitor_read_config (plugin, global);
-    
+
     g_signal_connect (plugin, "free-data", G_CALLBACK (monitor_free), global);
-    
+
     g_signal_connect (plugin, "save", G_CALLBACK (monitor_write_config), global);
-    
+
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin", G_CALLBACK (monitor_create_options), global);
-    
+
+    xfce_panel_plugin_menu_show_about (plugin);
+    g_signal_connect (plugin, "about", G_CALLBACK (monitor_show_about), global);
+
     g_signal_connect (plugin, "size-changed", G_CALLBACK (monitor_set_size), global);
-    
+
     g_signal_connect (plugin, "mode-changed", G_CALLBACK (monitor_set_mode), global);
-    
+
     gtk_container_add(GTK_CONTAINER(plugin), global->ebox);
-    
+
     run_update( global );
 }
 
